@@ -56,6 +56,8 @@
 #include <vstring.h>
 #include <vstream.h>
 #include <mymalloc.h>
+#include <stringops.h>
+#include <nvtable.h>
 
 /* Global library. */
 
@@ -104,6 +106,11 @@ void    cleanup_envelope(CLEANUP_STATE *state, int type, char *str, int len)
 
 static void cleanup_envelope_process(CLEANUP_STATE *state, int type, char *buf, int len)
 {
+    char   *attr_name;
+    char   *attr_value;
+    const char *error_text;
+    int     extra_flags;
+
     if (type == REC_TYPE_MESG) {
 	if (state->sender == 0 || state->time == 0) {
 	    msg_warn("%s: missing sender or time envelope record",
@@ -119,6 +126,14 @@ static void cleanup_envelope_process(CLEANUP_STATE *state, int type, char *buf,
 	}
 	return;
     }
+    if (type == REC_TYPE_FLGS) {
+	extra_flags = atol(buf);
+	if (extra_flags & ~CLEANUP_FLAG_MASK_EXTRA)
+	    msg_warn("%s: bad extra flags: 0x%x", state->queue_id, extra_flags);
+	else
+	    state->flags |= extra_flags;
+	return;
+    }
     if (strchr(REC_TYPE_ENVELOPE, type) == 0) {
 	msg_warn("%s: unexpected record type %d in envelope",
 		 state->queue_id, type);
@@ -196,6 +211,21 @@ static void cleanup_envelope_process(CLEANUP_STATE *state, int type, char *buf,
 	    state->errs |= CLEANUP_STAT_BAD;
 	    return;
 	}
+    } else if (type == REC_TYPE_ATTR) {
+	if (state->attr->used >= var_qattr_count_limit) {
+	    msg_warn("%s: queue file attribute count exceeds safety limit: %d",
+		     state->queue_id, var_qattr_count_limit);
+	    state->errs |= CLEANUP_STAT_BAD;
+	    return;
+	}
+	cleanup_out(state, type, buf, len);
+	if ((error_text = split_nameval(buf, &attr_name, &attr_value)) != 0) {
+	    msg_warn("%s: malformed attribute: %s: %.100s",
+		     state->queue_id, error_text, buf);
+	    state->errs |= CLEANUP_STAT_BAD;
+	    return;
+	}
+	nvtable_update(state->attr, attr_name, attr_value);
     } else {
 	cleanup_out(state, type, buf, len);
     }
