@@ -51,6 +51,7 @@
 #include <vstream.h>
 #include <argv.h>
 #include <mymalloc.h>
+#include <nvtable.h>
 
 /* Global library. */
 
@@ -59,6 +60,7 @@
 #include <rec_type.h>
 #include <mail_params.h>
 #include <ext_prop.h>
+#include <mail_proto.h>
 
 /* Application-specific. */
 
@@ -72,12 +74,31 @@ static void cleanup_extracted_process(CLEANUP_STATE *, int, char *, int);
 
 void    cleanup_extracted(CLEANUP_STATE *state, int type, char *buf, int len)
 {
+    const char *encoding;
 
     /*
      * Start the extracted segment.
      */
     cleanup_out_string(state, REC_TYPE_XTRA, "");
 
+    /*
+     * Put the optional content filter before the mandatory Return-Receipt-To
+     * and Errors-To so that the queue manager will pick up the filter name
+     * before starting deliveries.
+     */
+    if (state->filter != 0)
+	cleanup_out_string(state, REC_TYPE_FILT, state->filter);
+
+    /*
+     * Older Postfix versions didn't emit encoding information, so this
+     * record can only be optional. Putting this before the mandatory
+     * Return-Receipt-To and Errors-To ensures that the queue manager will
+     * pick up the content encoding before starting deliveries.
+     */
+    if ((encoding = nvtable_find(state->attr, MAIL_ATTR_ENCODING)) != 0)
+	cleanup_out_format(state, REC_TYPE_ATTR, "%s=%s",
+			   MAIL_ATTR_ENCODING, encoding);
+
     /*
      * Always emit Return-Receipt-To and Errors-To records, and always emit
      * them ahead of extracted recipients, so that the queue manager does not
@@ -99,21 +120,13 @@ void    cleanup_extracted(CLEANUP_STATE *state, int type, char *buf, int len)
 
 /* cleanup_extracted_process - process extracted segment */
 
-static void cleanup_extracted_process(CLEANUP_STATE *state, int type, char *buf, int unused_len)
+static void cleanup_extracted_process(CLEANUP_STATE *state, int type, char *buf, int len)
 {
     char   *myname = "cleanup_extracted_process";
     VSTRING *clean_addr;
     ARGV   *rcpt;
     char  **cpp;
 
-    if (type == REC_TYPE_RRTO) {
-	/* XXX Use extracted information instead. */
-	return;
-    }
-    if (type == REC_TYPE_ERTO) {
-	/* XXX Use extracted information instead. */
-	return;
-    }
     if (type == REC_TYPE_RCPT) {
 	clean_addr = vstring_alloc(100);
 	cleanup_rewrite_internal(clean_addr, *buf ? buf : var_empty_addr);
@@ -133,9 +146,7 @@ static void cleanup_extracted_process(CLEANUP_STATE *state, int type, char *buf,
 	return;
     }
     if (type != REC_TYPE_END) {
-	msg_warn("%s: unexpected record type %d in extracted segment",
-		 state->queue_id, type);
-	state->errs |= CLEANUP_STAT_BAD;
+	cleanup_out(state, type, buf, len);
 	return;
     }
 
