@@ -23,11 +23,37 @@
 /*	const char *format;
 /*	va_list ap;
 /*
-/*	int	bounce_flush(flags, queue, id, sender)
+/*	int	bounce_flush(flags, queue, id, encoding, sender)
 /*	int	flags;
 /*	const char *queue;
 /*	const char *id;
+/*	const char *encoding;
 /*	const char *sender;
+/*
+/*	int	bounce_one(flags, queue, id, encoding, sender,
+/*				recipient, relay, entry, format, ...)
+/*	int	flags;
+/*	const char *queue;
+/*	const char *id;
+/*	const char *encoding;
+/*	const char *sender;
+/*	const char *recipient;
+/*	const char *relay;
+/*	time_t	entry;
+/*	const char *format;
+/*
+/*	int	vbounce_one(flags, queue, id, encoding, sender,
+/*				recipient, relay, entry, format, ap)
+/*	int	flags;
+/*	const char *queue;
+/*	const char *id;
+/*	const char *encoding;
+/*	const char *sender;
+/*	const char *recipient;
+/*	const char *relay;
+/*	time_t	entry;
+/*	const char *format;
+/*	va_list ap;
 /* DESCRIPTION
 /*	This module implements the client interface to the message
 /*	bounce service, which maintains a per-message log of status
@@ -42,6 +68,15 @@
 /*	the specified sender, including the bounce log that was
 /*	built with bounce_append().
 /*
+/*	bounce_one() bounces one recipient and immediately sends a 
+/*	notification to the sender. This procedure does not append 
+/*	the recipient and reason to the per-message bounce log, and 
+/*	should be used when a delivery agent changes the error 
+/*	return address in a manner that depends on the recipient
+/*	address.
+/*
+/*	vbounce_one() implements an alternative interface.
+/*
 /*	Arguments:
 /* .IP flags
 /*	The bitwise OR of zero or more of the following (specify
@@ -56,6 +91,8 @@
 /* .IP id
 /*	The message queue id if the original message file. The bounce log
 /*	file has the same name as the original message file.
+/* .IP encoding
+/*	The body content encoding: MAIL_ATTR_ENC_{7BIT,8BIT,NONE}.
 /* .IP sender
 /*	The sender envelope address.
 /* .IP relay
@@ -143,7 +180,7 @@ int     vbounce_append(int flags, const char *id, const char *recipient,
     delay = time((time_t *) 0) - entry;
     vstring_vsprintf(why, fmt, ap);
     if (mail_command_client(MAIL_CLASS_PRIVATE, var_soft_bounce ?
-			    MAIL_SERVICE_DEFER : MAIL_SERVICE_BOUNCE,
+			    var_defer_service : var_bounce_service,
 			    ATTR_TYPE_NUM, MAIL_ATTR_NREQ, BOUNCE_CMD_APPEND,
 			    ATTR_TYPE_NUM, MAIL_ATTR_FLAGS, flags,
 			    ATTR_TYPE_STR, MAIL_ATTR_QUEUEID, id,
@@ -156,7 +193,7 @@ int     vbounce_append(int flags, const char *id, const char *recipient,
 		 vstring_str(why));
 	status = (var_soft_bounce ? -1 : 0);
     } else if ((flags & BOUNCE_FLAG_CLEAN) == 0) {
-	status = defer_append(flags, id, recipient, "bounce", delay,
+	status = defer_append(flags, id, recipient, "bounce", entry,
 			      "bounce failed");
     } else {
 	status = -1;
@@ -168,7 +205,7 @@ int     vbounce_append(int flags, const char *id, const char *recipient,
 /* bounce_flush - flush the bounce log and deliver to the sender */
 
 int     bounce_flush(int flags, const char *queue, const char *id,
-		             const char *sender)
+		             const char *encoding, const char *sender)
 {
 
     /*
@@ -177,11 +214,12 @@ int     bounce_flush(int flags, const char *queue, const char *id,
      */
     if (var_soft_bounce)
 	return (-1);
-    if (mail_command_client(MAIL_CLASS_PRIVATE, MAIL_SERVICE_BOUNCE,
+    if (mail_command_client(MAIL_CLASS_PRIVATE, var_bounce_service,
 			    ATTR_TYPE_NUM, MAIL_ATTR_NREQ, BOUNCE_CMD_FLUSH,
 			    ATTR_TYPE_NUM, MAIL_ATTR_FLAGS, flags,
 			    ATTR_TYPE_STR, MAIL_ATTR_QUEUE, queue,
 			    ATTR_TYPE_STR, MAIL_ATTR_QUEUEID, id,
+			    ATTR_TYPE_STR, MAIL_ATTR_ENCODING, encoding,
 			    ATTR_TYPE_STR, MAIL_ATTR_SENDER, sender,
 			    ATTR_TYPE_END) == 0) {
 	return (0);
@@ -192,3 +230,64 @@ int     bounce_flush(int flags, const char *queue, const char *id,
 	return (-1);
     }
 }
+
+/* bounce_one - send notice for one recipient */
+
+int     bounce_one(int flags, const char *queue, const char *id,
+		           const char *encoding, const char *sender,
+		           const char *recipient, const char *relay,
+		           time_t entry, const char *fmt,...)
+{
+    va_list ap;
+    int     status;
+
+    va_start(ap, fmt);
+    status = vbounce_one(flags, queue, id, encoding, sender,
+			 recipient, relay, entry, fmt, ap);
+    va_end(ap);
+    return (status);
+}
+
+/* vbounce_one - send notice for one recipient */
+
+int     vbounce_one(int flags, const char *queue, const char *id,
+		            const char *encoding, const char *sender,
+		            const char *recipient, const char *relay,
+		            time_t entry, const char *fmt, va_list ap)
+{
+    VSTRING *why;
+    int     status;
+    int     delay;
+
+    /*
+     * When we're not bouncing, then use the standard logfile based
+     * procedure.
+     */
+    if (var_soft_bounce)
+	return (vbounce_append(flags, id, recipient, relay, entry, fmt, ap));
+
+    why = vstring_alloc(100);
+    delay = time((time_t *) 0) - entry;
+    vstring_vsprintf(why, fmt, ap);
+    if (mail_command_client(MAIL_CLASS_PRIVATE, var_bounce_service,
+			    ATTR_TYPE_NUM, MAIL_ATTR_NREQ, BOUNCE_CMD_ONE,
+			    ATTR_TYPE_NUM, MAIL_ATTR_FLAGS, flags,
+			    ATTR_TYPE_STR, MAIL_ATTR_QUEUE, queue,
+			    ATTR_TYPE_STR, MAIL_ATTR_QUEUEID, id,
+			    ATTR_TYPE_STR, MAIL_ATTR_ENCODING, encoding,
+			    ATTR_TYPE_STR, MAIL_ATTR_SENDER, sender,
+			    ATTR_TYPE_STR, MAIL_ATTR_RECIP, recipient,
+			    ATTR_TYPE_STR, MAIL_ATTR_WHY, vstring_str(why),
+			    ATTR_TYPE_END) == 0) {
+	msg_info("%s: to=<%s>, relay=%s, delay=%d, status=bounced (%s)",
+		 id, recipient, relay, delay, vstring_str(why));
+	status = 0;
+    } else if ((flags & BOUNCE_FLAG_CLEAN) == 0) {
+	status = defer_append(flags, id, recipient, "bounce", entry,
+			      "bounce failed");
+    } else {
+	status = -1;
+    }
+    vstring_free(why);
+    return (status);
+}
