@@ -133,7 +133,7 @@ void    resolve_clnt_query(const char *addr, RESOLVE_REPLY *reply)
     /*
      * Peek at the cache.
      */
-    if (strcmp(addr, STR(last_addr)) == 0) {
+    if (*addr && strcmp(addr, STR(last_addr)) == 0) {
 	vstring_strcpy(reply->transport, STR(last_reply.transport));
 	vstring_strcpy(reply->nexthop, STR(last_reply.nexthop));
 	vstring_strcpy(reply->recipient, STR(last_reply.recipient));
@@ -152,7 +152,7 @@ void    resolve_clnt_query(const char *addr, RESOLVE_REPLY *reply)
      */
     if (rewrite_clnt_stream == 0)
 	rewrite_clnt_stream = clnt_stream_create(MAIL_CLASS_PRIVATE,
-				  MAIL_SERVICE_REWRITE, var_ipc_idle_limit);
+				  var_rewrite_service, var_ipc_idle_limit);
 
     for (;;) {
 	stream = clnt_stream_access(rewrite_clnt_stream);
@@ -179,7 +179,7 @@ void    resolve_clnt_query(const char *addr, RESOLVE_REPLY *reply)
 			 STR(reply->nexthop), STR(reply->recipient));
 	    if (STR(reply->transport)[0] == 0)
 		msg_warn("%s: null transport result for: <%s>", myname, addr);
-	    else if (STR(reply->recipient)[0] == 0)
+	    else if (STR(reply->recipient)[0] == 0 && *addr != 0)
 		msg_warn("%s: null recipient result for: <%s>", myname, addr);
 	    else
 		break;
@@ -230,7 +230,7 @@ static void resolve(char *addr, RESOLVE_REPLY *reply)
     vstream_fflush(VSTREAM_OUT);
 }
 
-main(int argc, char **argv)
+int     main(int argc, char **argv)
 {
     RESOLVE_REPLY reply;
     int     ch;
@@ -264,7 +264,9 @@ main(int argc, char **argv)
 	while (vstring_fgets_nonl(buffer, VSTREAM_IN)) {
 	    resolve(STR(buffer), &reply);
 	}
+	vstring_free(buffer);
     }
+    resolve_clnt_free(&reply);
 }
 
 #endif
