@@ -435,6 +435,7 @@
 #include <been_here.h>
 #include <mail_params.h>
 #include <ext_prop.h>
+#include <maps.h>
 
 /* Single server skeleton. */
 
@@ -478,6 +479,7 @@ int     local_file_deliver_mask;
 int     local_ext_prop_mask;
 int     local_deliver_hdr_mask;
 int     local_mbox_lock_mask;
+MAPS   *alias_maps;
 
 /* local_deliver - deliver message with extreme prejudice */
 
@@ -514,6 +516,7 @@ static int local_deliver(DELIVER_REQUEST *rqst, char *service)
     state.msg_attr.queue_id = rqst->queue_id;
     state.msg_attr.fp = rqst->fp;
     state.msg_attr.offset = rqst->data_offset;
+    state.msg_attr.encoding = rqst->encoding;
     state.msg_attr.sender = rqst->sender;
     state.msg_attr.relay = service;
     state.msg_attr.arrival_time = rqst->arrival_time;
@@ -661,6 +664,7 @@ static void pre_init(char *unused_name, char **unused_argv)
 		      VAR_MAILBOX_LIMIT, VAR_MESSAGE_LIMIT);
 	set_file_limit(var_mailbox_limit);
     }
+    alias_maps = maps_create("aliases", var_alias_maps, DICT_FLAG_LOCK);
 }
 
 /* main - pass control to the single-threaded skeleton */
