@@ -122,6 +122,7 @@
 #include <opened.h>
 #include <resolve_local.h>
 #include <verp_sender.h>
+#include <mail_proto.h>
 
 /* Client stubs. */
 
@@ -153,6 +154,7 @@ static QMGR_MESSAGE *qmgr_message_create(const char *queue_name,
     message->data_offset = 0;
     message->queue_id = mystrdup(queue_id);
     message->queue_name = mystrdup(queue_name);
+    message->encoding = 0;
     message->sender = 0;
     message->errors_to = 0;
     message->return_receipt = 0;
@@ -283,6 +285,9 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
     long    save_offset = message->rcpt_offset;	/* save a flag */
     char   *start;
     int     recipient_limit;
+    const char *error_text;
+    char   *name;
+    char   *value;
 
     /*
      * Initialize. No early returns or we have a memory leak.
@@ -373,11 +378,13 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 	    if (message->arrival_time == 0)
 		message->arrival_time = atol(start);
 	} else if (rec_type == REC_TYPE_FILT) {
-	    if (message->filter_xport == 0)
-		message->filter_xport = mystrdup(start);
+	    if (message->filter_xport != 0)
+		myfree(message->filter_xport);
+	    message->filter_xport = mystrdup(start);
 	} else if (rec_type == REC_TYPE_INSP) {
-	    if (message->inspect_xport == 0)
-		message->inspect_xport = mystrdup(start);
+	    if (message->inspect_xport != 0)
+		myfree(message->inspect_xport);
+	    message->inspect_xport = mystrdup(start);
 	} else if (rec_type == REC_TYPE_FROM) {
 	    if (message->sender == 0) {
 		message->sender = mystrdup(start);
@@ -404,6 +411,18 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 			break;
 		}
 	    }
+	} else if (rec_type == REC_TYPE_ATTR) {
+	    if ((error_text = split_nameval(start, &name, &value)) != 0) {
+		msg_warn("%s: bad attribute: %s: %.200s",
+			 message->queue_id, error_text, start);
+		break;
+	    }
+	    /* Allow extra segment to override envelope segment info. */
+	    if (strcmp(name, MAIL_ATTR_ENCODING) == 0) {
+		if (message->encoding != 0)
+		    myfree(message->encoding);
+		message->encoding = mystrdup(value);
+	    }
 	} else if (rec_type == REC_TYPE_ERTO) {
 	    if (message->errors_to == 0) {
 		message->errors_to = mystrdup(start);
@@ -447,6 +466,8 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 	message->errors_to = mystrdup("");
     if (message->return_receipt == 0)
 	message->return_receipt = mystrdup("");
+    if (message->encoding == 0)
+	message->encoding = mystrdup(MAIL_ATTR_ENC_NONE);
 
     /*
      * Clean up.
@@ -617,7 +638,12 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	 * Resolve the destination to (transport, nexthop, address). The
 	 * result address may differ from the one specified by the sender.
 	 */
-	resolve_clnt_query(recipient->address, &reply);
+	if (var_sender_routing == 0) {
+	    resolve_clnt_query(recipient->address, &reply);
+	} else {
+	    resolve_clnt_query(message->sender, &reply);
+	    vstring_strcpy(reply.recipient, recipient->address);
+	}
 	if (message->filter_xport) {
 	    vstring_strcpy(reply.transport, message->filter_xport);
 	    if ((nexthop = split_at(STR(reply.transport), ':')) == 0
@@ -628,6 +654,11 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	    if (!STREQ(recipient->address, STR(reply.recipient)))
 		UPDATE(recipient->address, STR(reply.recipient));
 	}
+	if (recipient->address[0] == 0) {
+	    qmgr_bounce_recipient(message, recipient,
+				  "null recipient address");
+	    continue;
+	}
 
 	/*
 	 * XXX The nexthop destination is also used as lookup key for the
@@ -926,6 +957,8 @@ void    qmgr_message_free(QMGR_MESSAGE *message)
 	qmgr_job_free(job);
     myfree(message->queue_id);
     myfree(message->queue_name);
+    if (message->encoding)
+	myfree(message->encoding);
     if (message->sender)
 	myfree(message->sender);
     if (message->verp_delims)
