@@ -113,6 +113,7 @@
 #include <opened.h>
 #include <resolve_local.h>
 #include <verp_sender.h>
+#include <mail_proto.h>
 
 /* Client stubs. */
 
@@ -143,6 +144,7 @@ static QMGR_MESSAGE *qmgr_message_create(const char *queue_name,
     message->data_offset = 0;
     message->queue_id = mystrdup(queue_id);
     message->queue_name = mystrdup(queue_name);
+    message->encoding = 0;
     message->sender = 0;
     message->errors_to = 0;
     message->return_receipt = 0;
@@ -203,6 +205,9 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
     char   *start;
     struct stat st;
     int     nrcpt = 0;
+    const char *error_text;
+    char   *name;
+    char   *value;
 
     /*
      * Initialize. No early returns or we have a memory leak.
@@ -254,11 +259,13 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 	    if (message->arrival_time == 0)
 		message->arrival_time = atol(start);
 	} else if (rec_type == REC_TYPE_FILT) {
-	    if (message->filter_xport == 0)
-		message->filter_xport = mystrdup(start);
+	    if (message->filter_xport != 0)
+		myfree(message->filter_xport);
+	    message->filter_xport = mystrdup(start);
 	} else if (rec_type == REC_TYPE_INSP) {
-	    if (message->inspect_xport == 0)
-		message->inspect_xport = mystrdup(start);
+	    if (message->inspect_xport != 0)
+		myfree(message->inspect_xport);
+	    message->inspect_xport = mystrdup(start);
 	} else if (rec_type == REC_TYPE_FROM) {
 	    if (message->sender == 0) {
 		message->sender = mystrdup(start);
@@ -294,6 +301,18 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 	    }
 	    if (vstream_fseek(message->fp, extra_offset, SEEK_SET) < 0)
 		msg_fatal("seek file %s: %m", VSTREAM_PATH(message->fp));
+	} else if (rec_type == REC_TYPE_ATTR) {
+	    if ((error_text = split_nameval(start, &name, &value)) != 0) {
+		msg_warn("%s: bad attribute: %s: %.200s",
+			 message->queue_id, error_text, start);
+		break;
+	    }
+	    /* Allow extra segment to override envelope segment info. */
+	    if (strcmp(name, MAIL_ATTR_ENCODING) == 0) {
+		if (message->encoding != 0)
+		    myfree(message->encoding);
+		message->encoding = mystrdup(value);
+	    }
 	} else if (rec_type == REC_TYPE_ERTO) {
 	    if (message->errors_to == 0)
 		message->errors_to = mystrdup(start);
@@ -336,6 +355,8 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 	message->errors_to = mystrdup("");
     if (message->return_receipt == 0)
 	message->return_receipt = mystrdup("");
+    if (message->encoding == 0)
+	message->encoding = mystrdup(MAIL_ATTR_ENC_NONE);
 
     /*
      * Clean up.
@@ -497,7 +518,12 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	 * Resolve the destination to (transport, nexthop, address). The
 	 * result address may differ from the one specified by the sender.
 	 */
-	resolve_clnt_query(recipient->address, &reply);
+	if (var_sender_routing == 0) {
+	    resolve_clnt_query(recipient->address, &reply);
+	} else {
+	    resolve_clnt_query(message->sender, &reply);
+	    vstring_strcpy(reply.recipient, recipient->address);
+	}
 	if (message->filter_xport) {
 	    vstring_strcpy(reply.transport, message->filter_xport);
 	    if ((nexthop = split_at(STR(reply.transport), ':')) == 0
@@ -508,6 +534,11 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	    if (!STREQ(recipient->address, STR(reply.recipient)))
 		UPDATE(recipient->address, STR(reply.recipient));
 	}
+	if (recipient->address[0] == 0) {
+	    qmgr_bounce_recipient(message, recipient,
+				  "null recipient address");
+	    continue;
+	}
 
 	/*
 	 * XXX The nexthop destination is also used as lookup key for the
@@ -691,23 +722,6 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	    continue;
 	}
 
-	/*
-	 * This queue is a hog. Defer this recipient until the queue drains.
-	 * When a site accumulates a large backlog, Postfix will deliver a
-	 * little chunk and hammer the disk as it defers the remainder of the
-	 * backlog and searches the deferred queue for deliverable mail.
-	 */
-	if (var_qmgr_hog < 100) {
-	    if (queue->todo_refcount + queue->busy_refcount
-		> (var_qmgr_hog / 100.0)
-		* (qmgr_recipient_count > 0.8 * var_qmgr_rcpt_limit ?
-		   qmgr_message_count : var_qmgr_active_limit)) {
-		qmgr_defer_recipient(message, recipient->address,
-				     "site destination queue overflow");
-		continue;
-	    }
-	}
-
 	/*
 	 * This queue is alive. Bind this recipient to this queue instance.
 	 */
@@ -761,6 +775,8 @@ void    qmgr_message_free(QMGR_MESSAGE *message)
 	msg_panic("qmgr_message_free: queue file is open");
     myfree(message->queue_id);
     myfree(message->queue_name);
+    if (message->encoding)
+	myfree(message->encoding);
     if (message->sender)
 	myfree(message->sender);
     if (message->verp_delims)
