@@ -103,8 +103,7 @@
 /* .IP \fBalways_bcc\fR
 /*	Address to send a copy of each message that enters the system.
 /* .IP \fBcommand_directory\fR
-/*	Location of Postfix support commands (default:
-/*	\fB$program_directory\fR).
+/*	Location of Postfix support commands.
 /* .IP \fBdebug_peer_level\fR
 /*	Increment in verbose logging level when a remote host matches a
 /*	pattern in the \fBdebug_peer_list\fR parameter.
@@ -199,6 +198,9 @@
 /* .IP \fBsmtpd_etrn_restrictions\fR
 /*	Restrict what domain names can be used in \fBETRN\fR commands,
 /*	and what clients may issue \fBETRN\fR commands.
+/* .IP \fBsmtpd_data_restrictions\fR
+/*	Restrictions on the \fBDATA\fR command. Currently, the only restriction
+/*	that makes sense here is \fBreject_unauth_pipelining\fR.
 /* .IP \fBallow_untrusted_routing\fR
 /*	Allow untrusted clients to specify addresses with sender-specified
 /*	routing.  Enabling this opens up nasty relay loopholes involving
@@ -224,6 +226,9 @@
 /* .fi
 /* .IP \fBaccess_map_reject_code\fR
 /*	Server response when a client violates an access database restriction.
+/* .IP \fBdefer_code\fR
+/*	Server response when a client request is rejected by the \fBdefer\fR
+/*	restriction.
 /* .IP \fBinvalid_hostname_reject_code\fR
 /*	Server response when a client violates the \fBreject_invalid_hostname\fR
 /*	restriction.
@@ -312,6 +317,8 @@
 #include <tok822.h>
 #include <verp_sender.h>
 #include <string_list.h>
+#include <quote_822_local.h>
+#include <lex_822.h>
 
 /* Single-threaded server skeleton. */
 
@@ -346,6 +353,7 @@ char   *var_helo_checks;
 char   *var_mail_checks;
 char   *var_rcpt_checks;
 char   *var_etrn_checks;
+char   *var_data_checks;
 int     var_unk_client_code;
 int     var_bad_name_code;
 int     var_unk_name_code;
@@ -356,6 +364,7 @@ int     var_access_map_code;
 char   *var_maps_rbl_domains;
 int     var_helo_required;
 int     var_reject_code;
+int     var_defer_code;
 int     var_smtpd_err_sleep;
 int     var_non_fqdn_code;
 char   *var_always_bcc;
@@ -532,13 +541,13 @@ static void mail_open_stream(SMTPD_STATE *state)
      */
     if (SMTPD_STAND_ALONE(state) == 0) {
 	state->dest = mail_stream_service(MAIL_CLASS_PUBLIC,
-					  MAIL_SERVICE_CLEANUP);
+					  var_cleanup_service);
 	if (state->dest == 0
 	    || attr_print(state->dest->stream, ATTR_FLAG_NONE,
 			ATTR_TYPE_NUM, MAIL_ATTR_FLAGS, CLEANUP_FLAG_FILTER,
 			  ATTR_TYPE_END) != 0)
 	    msg_fatal("unable to connect to the %s %s service",
-		      MAIL_CLASS_PUBLIC, MAIL_SERVICE_CLEANUP);
+		      MAIL_CLASS_PUBLIC, var_cleanup_service);
     }
 
     /*
@@ -663,6 +672,7 @@ static int mail_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *argv)
     int     narg;
     char   *arg;
     char   *verp_delims = 0;
+    char   *encoding = 0;
 
     state->msg_size = 0;
 
@@ -703,10 +713,11 @@ static int mail_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *argv)
     }
     for (narg = 3; narg < argc; narg++) {
 	arg = argv[narg].strval;
-	if (strcasecmp(arg, "BODY=8BITMIME") == 0
-	    || strcasecmp(arg, "BODY=7BIT") == 0) {
-	     /* void */ ;
-	} else if (strncasecmp(arg, "SIZE=", 5) == 0) {
+	if (strcasecmp(arg, "BODY=8BITMIME") == 0) {	/* RFC 1652 */
+	    encoding = MAIL_ATTR_ENC_8BIT;
+	} else if (strcasecmp(arg, "BODY=7BIT") == 0) {	/* RFC 1652 */
+	    encoding = MAIL_ATTR_ENC_7BIT;
+	} else if (strncasecmp(arg, "SIZE=", 5) == 0) {	/* RFC 1870 */
 	    /* Reject non-numeric size. */
 	    if (!alldig(arg + 5)) {
 		state->error_mask |= MAIL_ERROR_PROTOCOL;
@@ -779,6 +790,20 @@ static int mail_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *argv)
     if (*var_filter_xport)
 	rec_fprintf(state->cleanup, REC_TYPE_FILT, "%s", var_filter_xport);
     rec_fputs(state->cleanup, REC_TYPE_FROM, argv[2].strval);
+    if (encoding != 0)
+	rec_fprintf(state->cleanup, REC_TYPE_ATTR, "%s=%s",
+		    MAIL_ATTR_ENCODING, encoding);
+    if (SMTPD_STAND_ALONE(state) == 0) {
+	rec_fprintf(state->cleanup, REC_TYPE_ATTR, "%s=%s",
+		    MAIL_ATTR_CLIENT_NAME, state->name);
+	rec_fprintf(state->cleanup, REC_TYPE_ATTR, "%s=%s",
+		    MAIL_ATTR_CLIENT_ADDR, state->addr);
+	rec_fprintf(state->cleanup, REC_TYPE_ATTR, "%s=%s",
+		    MAIL_ATTR_ORIGIN, state->namaddr);
+	if (state->helo_name != 0)
+	    rec_fprintf(state->cleanup, REC_TYPE_ATTR, "%s=%s",
+			MAIL_ATTR_HELO_NAME, state->helo_name);
+    }
     if (verp_delims)
 	rec_fputs(state->cleanup, REC_TYPE_VERP, verp_delims);
     state->sender = mystrdup(argv[2].strval);
@@ -904,6 +929,7 @@ static void rcpt_reset(SMTPD_STATE *state)
 
 static int data_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *unused_argv)
 {
+    char   *err;
     char   *start;
     int     len;
     int     curr_rec_type;
@@ -930,6 +956,10 @@ static int data_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *unused_argv)
 	smtpd_chat_reply(state, "501 Syntax: DATA");
 	return (-1);
     }
+    if (SMTPD_STAND_ALONE(state) == 0 && (err = smtpd_check_data(state)) != 0) {
+	smtpd_chat_reply(state, "%s", err);
+	return (-1);
+    }
 
     /*
      * Terminate the message envelope segment. Start the message content
@@ -948,9 +978,9 @@ static int data_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *unused_argv)
 		    "\tby %s (%s) with %s id %s",
 		    var_myhostname, var_mail_name,
 		    state->protocol, state->queue_id);
-	/* XXX Should RFC 822 externalize recipient address */
+	quote_822_local(state->buffer, state->recipient);
 	rec_fprintf(state->cleanup, REC_TYPE_NORM,
-		"\tfor <%s>; %s", state->recipient, mail_date(state->time));
+	      "\tfor <%s>; %s", STR(state->buffer), mail_date(state->time));
     } else {
 	rec_fprintf(state->cleanup, REC_TYPE_NORM,
 		    "\tby %s (%s) with %s",
@@ -959,9 +989,9 @@ static int data_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *unused_argv)
 		    "\tid %s; %s", state->queue_id, mail_date(state->time));
     }
 #ifdef RECEIVED_ENVELOPE_FROM
-    /* XXX Should RFC 822 externalize sender address */
+    quote_822_local(state->buffer, state->sender);
     rec_fprintf(state->cleanup, REC_TYPE_NORM,
-		"\t(envelope-from %s)", state->sender);
+		"\t(envelope-from %s)", STR(state->buffer));
 #endif
     smtpd_chat_reply(state, "354 End data with <CR><LF>.<CR><LF>");
 
@@ -994,7 +1024,7 @@ static int data_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *unused_argv)
 		continue;
 	    }
 	    first = 0;
-	    if (len > 0 && ISSPACE(start[0]))
+	    if (len > 0 && IS_SPACE_TAB(start[0]))
 		rec_put(state->cleanup, REC_TYPE_NORM, "", 0);
 	}
 	if (prev_rec_type != REC_TYPE_CONT
@@ -1323,6 +1353,8 @@ static SMTPD_CMD smtpd_cmd_table[] = {
     "ETRN", etrn_cmd, SMTPD_CMD_FLAG_LIMIT,
     "QUIT", quit_cmd, 0,
     "Received:", 0, SMTPD_CMD_FLAG_HEADER,
+    "Reply-To:", 0, SMTPD_CMD_FLAG_HEADER,
+    "Message-ID:", 0, SMTPD_CMD_FLAG_HEADER,
     "Subject:", 0, SMTPD_CMD_FLAG_HEADER,
     "From:", 0, SMTPD_CMD_FLAG_HEADER,
     0,
@@ -1407,8 +1439,8 @@ static void smtpd_proto(SMTPD_STATE *state)
 		continue;
 	    }
 	    if (cmdp->flags & SMTPD_CMD_FLAG_HEADER) {
-		msg_warn("%s sent message header instead of SMTP command: %.100s",
-			 state->namaddr, vstring_str(state->buffer));
+		msg_warn("%s sent %s header instead of SMTP command: %.100s",
+		    state->namaddr, cmdp->name, vstring_str(state->buffer));
 		smtpd_chat_reply(state, "221 Error: I can break rules, too. Goodbye.");
 		break;
 	    }
@@ -1557,6 +1589,7 @@ int     main(int argc, char **argv)
 	VAR_MAPS_RBL_CODE, DEF_MAPS_RBL_CODE, &var_maps_rbl_code, 0, 0,
 	VAR_ACCESS_MAP_CODE, DEF_ACCESS_MAP_CODE, &var_access_map_code, 0, 0,
 	VAR_REJECT_CODE, DEF_REJECT_CODE, &var_reject_code, 0, 0,
+	VAR_DEFER_CODE, DEF_DEFER_CODE, &var_defer_code, 0, 0,
 	VAR_NON_FQDN_CODE, DEF_NON_FQDN_CODE, &var_non_fqdn_code, 0, 0,
 	VAR_SMTPD_JUNK_CMD, DEF_SMTPD_JUNK_CMD, &var_smtpd_junk_cmd_limit, 1, 0,
 	VAR_SMTPD_HIST_THRSH, DEF_SMTPD_HIST_THRSH, &var_smtpd_hist_thrsh, 1, 0,
@@ -1585,6 +1618,7 @@ int     main(int argc, char **argv)
 	VAR_MAIL_CHECKS, DEF_MAIL_CHECKS, &var_mail_checks, 0, 0,
 	VAR_RCPT_CHECKS, DEF_RCPT_CHECKS, &var_rcpt_checks, 0, 0,
 	VAR_ETRN_CHECKS, DEF_ETRN_CHECKS, &var_etrn_checks, 0, 0,
+	VAR_DATA_CHECKS, DEF_DATA_CHECKS, &var_data_checks, 0, 0,
 	VAR_MAPS_RBL_DOMAINS, DEF_MAPS_RBL_DOMAINS, &var_maps_rbl_domains, 0, 0,
 	VAR_ALWAYS_BCC, DEF_ALWAYS_BCC, &var_always_bcc, 0, 0,
 	VAR_ERROR_RCPT, DEF_ERROR_RCPT, &var_error_rcpt, 1, 0,
@@ -1597,7 +1631,7 @@ int     main(int argc, char **argv)
 	VAR_ALIAS_MAPS, DEF_ALIAS_MAPS, &var_alias_maps, 0, 0,
 	VAR_LOCAL_RCPT_MAPS, DEF_LOCAL_RCPT_MAPS, &var_local_rcpt_maps, 0, 0,
 	VAR_SMTPD_SASL_OPTS, DEF_SMTPD_SASL_OPTS, &var_smtpd_sasl_opts, 0, 0,
-	VAR_SMTPD_SASL_REALM, DEF_SMTPD_SASL_REALM, &var_smtpd_sasl_realm, 1, 0,
+	VAR_SMTPD_SASL_REALM, DEF_SMTPD_SASL_REALM, &var_smtpd_sasl_realm, 0, 0,
 	VAR_FILTER_XPORT, DEF_FILTER_XPORT, &var_filter_xport, 0, 0,
 	VAR_PERM_MX_NETWORKS, DEF_PERM_MX_NETWORKS, &var_perm_mx_networks, 0, 0,
 	VAR_SMTPD_SND_AUTH_MAPS, DEF_SMTPD_SND_AUTH_MAPS, &var_smtpd_snd_auth_maps, 0, 0,
